/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import java.awt.Graphics2D;
import java.awt.Image;
import java.util.ArrayList;
import javax.swing.JLabel;

/**
 *
 * @author Emerson Ribeiro de Mello
 */
public class Jogador extends Elemento {

    protected int cima; // código da tecla para mover para cima
    protected int baixo;// código da tecla para mover para baixo
    protected int direita;// código da tecla para mover para direita
    protected int esquerda;// código da tecla para mover para esquerda
    protected int atirar;// código da tecla para atirar
    private JLabel coordX; // JLabel no Frame principal para representar a coordenada X
    private JLabel coordY;// JLabel no Frame principal para representar a coordenada Y
    protected int vidas;
    protected int morrendo;
    ArrayList<Bomba> bombas;
    private boolean morrerPiscando;
    private int temBomba;
    protected int qntdBombas;
    protected int tamFogo;
    /* @param pai referencia a area de Desenho
     * @param imagemNome - Nome da imagem que sera' usada para representar o jogador
     * @param bombaNome  - Nome da imagem que sera' usada para representar a bomba deste jogador
     * @param mX Linha na matriz
     * @param mY Coluna na matriz
     * @param c codigo da tecla que sera' usado para mover para cima
     * @param b codigo da tecla que sera' usado para mover para baixo
     * @param d codigo da tecla que sera' usado para mover para esquerda
     * @param e codigo da tecla que sera' usado para mover para direita
     * @param a codigo da tecla que sera' usado para atirar
     */

    public Jogador(JogoTimer pai, String imagemNome,
            int mx, int my, int c, int b, int d, int e, int a) {
        super(pai, imagemNome, mx, my);
        this.icone = this.carregarImagem(this.imagemNome);
        this.cima = c;
        this.baixo = b;
        this.direita = d;
        this.esquerda = e;
        this.atirar = a;
        //this.coordX = jLx;
        //this.coordY = jLy;
        this.bombas = new ArrayList<Bomba>();
        this.vidas = 3;
        this.morrendo = 0;
        this.qntdBombas = 1;
        this.temBomba = 1;
        tamFogo = 1;
        
    }

    /**
     * Desenha a imagem no contexto gráfico do Pai
     * @param g 
     */
    @Override
    public void desenhar(Graphics2D g) {
        //desenhando as bombas
        for (Bomba bomba : bombas) {
            bomba.desenhar(g);

        }

        //se o tempo em que o jogador fica morto acabar
        //e o jogador não estiver desenhado, desenhe-o novamente
        if ((morrendo <= 0) || morrerPiscando == false) {
            // desenhando o icone do Jogador na Tela
            g.drawImage(this.icone, this.posX, this.posY, this.jogo);

        }

        // atualizando os jLabel com as coordenadas. Inuteis para um jogo real
        // mas a ideia poderia ser aproveitada para atualizar placar, etc.
       // this.atualizaRepresentacaoDasCoordenadasXY();
    }

    /**
     * Atualiza coordenadas, posicao na matriz e a bomba
     */
    @Override
    //funcao que atualiza os parametros do jogador
    public void atualizar() {
        int destinoLin = matLinha; // linha destino apos o movimento
        int destinoCol = matColuna; // coluna destino apos o movimento
        int destinoX = posX; // coordenada X destino na area de desenho
        int destinoY = posY; // coordenada Y destino na area de desenho
        //se as vidas de algum jogador acabar, pare o jogo
        if (vidas == 0) {
            this.jogo.finalizar();
        }

        //atualizando a posicao no cenario conforme as teclas de movimento sao 
        //pressionadas
        if (jogo.framePrincipal.teclado.teclaApertada(this.cima)) {
            destinoY = posY - this.velocidade;
            destinoLin--;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.baixo)) {
            destinoY = posY + this.velocidade;
            destinoLin++;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.esquerda)) {
            destinoX = posX - this.velocidade;
            destinoCol--;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.direita)) {
            destinoX = posX + this.velocidade;
            destinoCol++;
        }

        if (jogo.estaLivreBomba(destinoLin, destinoCol)) {
            this.jogo.poder = this.jogo.cenario[destinoLin][destinoCol];
            if (this.jogo.poder == 4) {
                qntdBombas++;
            }
            this.jogo.cenario[destinoLin][destinoCol] = 2;
        }
        if (jogo.estaLivreFogo(destinoLin, destinoCol)) {
            this.jogo.poder = this.jogo.cenario[destinoLin][destinoCol];
            if (this.jogo.poder == 5) {
                tamFogo++;
            }
            this.jogo.cenario[destinoLin][destinoCol] = 2;
        }
        if (jogo.estaLivreRato(destinoLin, destinoCol)) {
            this.jogo.poder = this.jogo.cenario[destinoLin][destinoCol];
            if (this.jogo.poder == 6 && tamFogo > 1) {
                tamFogo--;
            }
            this.jogo.cenario[destinoLin][destinoCol] = 2;
        }
        if (jogo.estaLivreGato(destinoLin, destinoCol)) {
            this.jogo.poder = this.jogo.cenario[destinoLin][destinoCol];
            if (this.jogo.poder == 7 && qntdBombas > 1) {
                qntdBombas--;
            }
            this.jogo.cenario[destinoLin][destinoCol] = 2;
        }
        if (jogo.estaLivreVida(destinoLin, destinoCol)) {
            this.jogo.poder = this.jogo.cenario[destinoLin][destinoCol];
            if (this.jogo.poder == 8) {
                vidas++;
            }
            this.jogo.cenario[destinoLin][destinoCol] = 2;
        }
        // se a celula destino estiver livre, mova o personagem para la'
        if (jogo.estaLivre(destinoLin, destinoCol)) {
            matLinha = destinoLin;
            matColuna = destinoCol;
            posX = destinoX;
            posY = destinoY;
        }


        //se o botao para colocar uma bomba for pressionado, crie uma bomba na ArrayList
        if ((jogo.framePrincipal.teclado.teclaApertadaPrimeiraVez(this.atirar))) {
            this.verificaSePodeNovaBomba();
            if (temBomba > 0) {
                Bomba b1 = new Bomba(this.jogo, "bomba2.png", this.matColuna, this.matLinha, this.tamFogo);
                bombas.add(b1);
                this.jogo.cenario[matLinha][matColuna] = 10;
                temBomba--;
            }

        }

        //percorrendo toda a lista para atualizar a posicao de cada bomba e
        //o seu tempo de vida
        for (Bomba bomba : bombas) {
            bomba.atualizar();
        }

        //se o jogador morrer, decremente o timer pouco a pouco ate zero e
        //fica alternando o boolean auxiliar para que na funcao desenhar
        //ele fique sendo desenhado e pare, fazendo o personagem piscar
        if (morrendo > 0) {
            morrendo -= 10;
            tamFogo = 1;
            if (morrerPiscando) {
                morrerPiscando = false;
            } else {
                morrerPiscando = true;

            }
        }
    }
    //verificando se o jogador nao estourou o seu limite de bombas
    //que eh por padrao 1

    public void verificaSePodeNovaBomba() {
        int numBomba = 0;//qtde bombas q tem na tela
        for (Bomba bomba : bombas) {
            if (bomba.bombaViva) {
                numBomba++;
            }
            //temBomba = bomba.bombaViva;
            //tembomba qtde bombas q posso ainda posso colocar
            //qndeBombas eh o numero total de bombas q posso colocar
            temBomba = qntdBombas - numBomba;

        }

    }

    /**
     * Atualiza os valores das coordenadas nos JLabel no Frame principal
     * Metodo usado so' para fins de depuracao
     */
    public void atualizaRepresentacaoDasCoordenadasXY() {
        this.coordX.setText("" + this.posX);
        this.coordY.setText("" + this.posY);
    }
}
